package org.eclipse.equinox.launcher;

import java.net.URLStreamHandlerFactory;
import java.util.NoSuchElementException;
import java.security.AllPermission;
import java.security.Permission;
import java.security.PermissionCollection;
import java.util.Enumeration;
import java.io.OutputStreamWriter;
import java.util.Calendar;
import java.io.Writer;
import java.io.PrintWriter;
import java.util.Date;
import java.util.zip.ZipEntry;
import java.io.FileNotFoundException;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.util.zip.ZipFile;
import java.io.OutputStream;
import java.util.Locale;
import java.io.InputStream;
import java.io.FileInputStream;
import java.net.MalformedURLException;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Vector;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.lang.reflect.Method;
import java.net.URLClassLoader;
import java.lang.reflect.InvocationTargetException;
import java.security.CodeSource;
import java.security.ProtectionDomain;
import java.security.Policy;
import com.jiuqi.util.ArrayUtils;
import com.jiuqi.util.BundleUtils;
import com.jiuqi.util.StringUtil;
import com.jiuqi.dna.launcher.ServerDaemon;
import java.io.BufferedWriter;
import java.io.File;
import java.util.Properties;
import java.net.URL;
import com.jiuqi.dna.launcher.Constants;

public class Main implements Constants
{
    protected boolean debug;
    protected String bootLocation;
    protected URL installLocation;
    protected URL configurationLocation;
    protected String parentConfigurationLocation;
    protected String framework;
    protected String devClassPath;
    private Properties devClassPathProps;
    protected boolean inDevelopmentMode;
    protected String os;
    protected String ws;
    protected String arch;
    private String library;
    private String exitData;
    private String vm;
    private String[] vmargs;
    private String[] commands;
    String[] extensionPaths;
    JNIBridge bridge;
    private boolean showSplash;
    private String splashLocation;
    private String endSplash;
    private boolean initialize;
    private boolean splashDown;
    private final Thread splashHandler;
    public static final String SPLASH_HANDLE = "org.eclipse.equinox.launcher.splash.handle";
    public static final String SPLASH_LOCATION = "org.eclipse.equinox.launcher.splash.location";
    private static final String FRAMEWORK = "-framework";
    private static final String INSTALL = "-install";
    private static final String INITIALIZE = "-initialize";
    private static final String VM = "-vm";
    private static final String VMARGS = "-vmargs";
    private static final String DEBUG = "-debug";
    private static final String DEV = "-dev";
    private static final String CONFIGURATION = "-configuration";
    private static final String NOSPLASH = "-nosplash";
    private static final String SHOWSPLASH = "-showsplash";
    private static final String EXITDATA = "-exitdata";
    private static final String NAME = "-name";
    private static final String LAUNCHER = "-launcher";
    private static final String LIBRARY = "--launcher.library";
    private static final String NL = "-nl";
    private static final String ENDSPLASH = "-endsplash";
    private static final String SPLASH_IMAGE = "splash.bmp";
    private static final String CLEAN = "-clean";
    private static final String NOEXIT = "-noExit";
    private static final String OS = "-os";
    private static final String WS = "-ws";
    private static final String REDIRECT = "-redirect";
    private static final String ARCH = "-arch";
    private static final String STARTUP = "-startup";
    private static final String OSGI = "org.eclipse.osgi";
    private static final String STARTER = "org.eclipse.core.runtime.adaptor.EclipseStarter";
    private static final String PLATFORM_URL = "platform:/base/";
    private static final String ECLIPSE_PROPERTIES = "eclipse.properties";
    private static final String FILE_SCHEME = "file:";
    protected static final String REFERENCE_SCHEME = "reference:";
    protected static final String JAR_SCHEME = "jar:";
    private static final String CONFIG_DIR = "configuration/";
    private static final String CONFIG_FILE = "config.ini";
    private static final String CONFIG_FILE_TEMP_SUFFIX = ".tmp";
    private static final String CONFIG_FILE_BAK_SUFFIX = ".bak";
    private static final String ECLIPSE = "eclipse";
    private static final String PRODUCT_SITE_MARKER = ".eclipseproduct";
    private static final String PRODUCT_SITE_ID = "id";
    private static final String PRODUCT_SITE_VERSION = "version";
    private static final String PROP_USER_HOME = "user.home";
    private static final String PROP_USER_DIR = "user.dir";
    private static final String PROP_INSTALL_AREA = "osgi.install.area";
    private static final String PROP_OS_TYPE = "os.type";
    private static final String PROP_REDIRECT_OUT = "out.redirect";
    private static final String PROP_CONSOLE_BLOCK = "osgi.console.blockOnReady";
    private static final String PROP_CONFIG_AREA = "osgi.configuration.area";
    private static final String PROP_CONFIG_AREA_DEFAULT = "osgi.configuration.area.default";
    private static final String PROP_BASE_CONFIG_AREA = "osgi.baseConfiguration.area";
    private static final String PROP_SHARED_CONFIG_AREA = "osgi.sharedConfiguration.area";
    private static final String PROP_CONFIG_CASCADED = "osgi.configuration.cascaded";
    protected static final String PROP_FRAMEWORK = "osgi.framework";
    private static final String PROP_SPLASHPATH = "osgi.splashPath";
    private static final String PROP_SPLASHLOCATION = "osgi.splashLocation";
    private static final String PROP_CLASSPATH = "osgi.frameworkClassPath";
    private static final String PROP_EXTENSIONS = "osgi.framework.extensions";
    private static final String PROP_FRAMEWORK_SYSPATH = "osgi.syspath";
    private static final String PROP_FRAMEWORK_SHAPE = "osgi.framework.shape";
    private static final String PROP_LOGFILE = "osgi.logfile";
    private static final String PROP_REQUIRED_JAVA_VERSION = "osgi.requiredJavaVersion";
    private static final String PROP_PARENT_CLASSLOADER = "osgi.parentClassloader";
    private static final String PROP_FRAMEWORK_PARENT_CLASSLOADER = "osgi.frameworkParentClassloader";
    private static final String PROP_NL = "osgi.nl";
    static final String PROP_NOSHUTDOWN = "osgi.noShutdown";
    private static final String PROP_DEBUG = "osgi.debug";
    private static final String PROP_EXITCODE = "eclipse.exitcode";
    private static final String PROP_EXITDATA = "eclipse.exitdata";
    private static final String PROP_VM = "eclipse.vm";
    private static final String PROP_VMARGS = "eclipse.vmargs";
    private static final String PROP_COMMANDS = "eclipse.commands";
    private static final String PROP_ECLIPSESECURITY = "eclipse.security";
    private static final String NONE = "@none";
    private static final String NO_DEFAULT = "@noDefault";
    private static final String USER_HOME = "@user.home";
    private static final String USER_DIR = "@user.dir";
    private static final String PARENT_CLASSLOADER_APP = "app";
    private static final String PARENT_CLASSLOADER_EXT = "ext";
    private static final String PARENT_CLASSLOADER_BOOT = "boot";
    private static final String PARENT_CLASSLOADER_CURRENT = "current";
    protected static final String SESSION = "!SESSION";
    protected static final String ENTRY = "!ENTRY";
    protected static final String MESSAGE = "!MESSAGE";
    protected static final String STACK = "!STACK";
    protected static final int ERROR = 4;
    protected static final String PLUGIN_ID = "com.jiuqi.dna.launcher";
    protected File logFile;
    protected BufferedWriter log;
    protected boolean newSession;
    private static final String[] bundleDirs;
    private static final String prefix2 = "reference:file:..";
    private static final String SUBFIX_START = "@start";
    private static final String[] notLoadBundles;
    
    static {
        bundleDirs = new String[] { "app", "dna", "thr", "lib", "bundles" };
        notLoadBundles = new String[] { "org.eclipse.osgi", "com.jiuqi.dna.launcher" };
    }
    
    public Main() {
        this.debug = false;
        this.bootLocation = null;
        this.installLocation = null;
        this.configurationLocation = null;
        this.parentConfigurationLocation = null;
        this.framework = "org.eclipse.osgi";
        this.devClassPath = null;
        this.devClassPathProps = null;
        this.inDevelopmentMode = false;
        this.os = null;
        this.ws = null;
        this.arch = null;
        this.library = null;
        this.exitData = null;
        this.vm = null;
        this.vmargs = null;
        this.commands = null;
        this.extensionPaths = null;
        this.bridge = null;
        this.showSplash = false;
        this.splashLocation = null;
        this.endSplash = null;
        this.initialize = false;
        this.splashDown = false;
        this.splashHandler = new SplashHandler();
        this.logFile = null;
        this.log = null;
        this.newSession = true;
    }
    
    private String getWS() {
        if (this.ws != null) {
            return this.ws;
        }
        final String os = this.getOS();
        if (os.equals("win32")) {
            return "win32";
        }
        if (os.equals("linux")) {
            return "gtk";
        }
        if (os.equals("macosx")) {
            return "carbon";
        }
        if (os.equals("hpux")) {
            return "motif";
        }
        if (os.equals("aix")) {
            return "motif";
        }
        if (os.equals("solaris")) {
            return "motif";
        }
        if (os.equals("qnx")) {
            return "photon";
        }
        return "unknown";
    }
    
    private String getOS() {
        if (this.os != null) {
            return this.os;
        }
        final String osName = System.getProperties().getProperty("os.name");
        if (osName.regionMatches(true, 0, "win32", 0, 3)) {
            return "win32";
        }
        if (osName.equalsIgnoreCase("SunOS")) {
            return "solaris";
        }
        if (osName.equalsIgnoreCase("Linux")) {
            return "linux";
        }
        if (osName.equalsIgnoreCase("QNX")) {
            return "qnx";
        }
        if (osName.equalsIgnoreCase("AIX")) {
            return "aix";
        }
        if (osName.equalsIgnoreCase("HP-UX")) {
            return "hpux";
        }
        if (osName.regionMatches(true, 0, "Mac OS", 0, "Mac OS".length())) {
            return "macosx";
        }
        return "unknown";
    }
    
    private String getArch() {
        if (this.arch != null) {
            return this.arch;
        }
        final String name = System.getProperties().getProperty("os.arch");
        if (name.equalsIgnoreCase("i386")) {
            return "x86";
        }
        if (name.equalsIgnoreCase("amd64")) {
            return "x86_64";
        }
        return name;
    }
    
    private void setupJNI(final URL[] defaultPath) {
        String libPath = null;
        if (this.library != null) {
            final File lib = new File(this.library);
            if (lib.isDirectory()) {
                libPath = this.searchFor("eclipse", lib.getAbsolutePath());
            }
            else if (lib.exists()) {
                libPath = lib.getAbsolutePath();
            }
        }
        if (libPath == null) {
            final String fragmentOS = this.getOS();
            final StringBuffer buffer = new StringBuffer("com.jiuqi.dna.launcher");
            buffer.append('.');
            buffer.append(this.getWS());
            buffer.append('.');
            buffer.append(fragmentOS);
            if (!fragmentOS.equals("macosx")) {
                buffer.append('.');
                buffer.append(this.getArch());
            }
            final String fragmentName = buffer.toString();
            String fragment = null;
            if (this.inDevelopmentMode) {
                final String devPathList = this.devClassPathProps.getProperty("com.jiuqi.dna.launcher");
                final String[] locations = this.getArrayFromList(devPathList);
                if (locations.length > 0) {
                    final File location = new File(locations[0]);
                    if (location.isAbsolute()) {
                        final String dir = location.getParent();
                        fragment = this.searchFor(fragmentName, dir);
                        if (fragment != null) {
                            libPath = this.searchFor("eclipse", fragment);
                        }
                    }
                }
            }
            if (libPath == null && this.bootLocation != null) {
                final URL[] urls = defaultPath;
                if (urls != null && urls.length > 0) {
                    for (int i = urls.length - 1; i >= 0 && libPath == null; --i) {
                        final File entryFile = new File(urls[i].getFile());
                        final String dir = entryFile.getParent();
                        if (this.inDevelopmentMode) {
                            final String devDir = String.valueOf(dir) + "/" + "com.jiuqi.dna.launcher" + "/fragments";
                            fragment = this.searchFor(fragmentName, devDir);
                        }
                        if (fragment == null) {
                            fragment = this.searchFor(fragmentName, dir);
                        }
                        if (fragment != null) {
                            libPath = this.searchFor("eclipse", fragment);
                        }
                    }
                }
            }
            if (libPath == null) {
                final URL install = this.getInstallLocation();
                String location2 = install.getFile();
                location2 = String.valueOf(location2) + "/bundles/";
                if (new File(location2).exists()) {
                    fragment = this.searchFor(fragmentName, location2);
                }
                else {
                    location2 = String.valueOf(location2) + "/thr/";
                    fragment = this.searchFor(fragmentName, location2);
                }
                if (fragment != null) {
                    libPath = this.searchFor("eclipse", fragment);
                }
            }
        }
        this.library = libPath;
        if (this.library != null) {
            this.bridge = new JNIBridge(this.library);
        }
    }
    
    protected void basicRun(final String[] args) throws Exception {
        System.getProperties().put("eclipse.startTime", Long.toString(System.currentTimeMillis()));
        final String[] args2 = { "-os", System.getProperty("os.name"), "-console" };
        this.commands = args2;
        final String[] passThruArgs = this.processCommandLine(args2);
        if (!this.debug) {
            this.debug = (System.getProperty("osgi.debug") != null);
        }
        this.setupVMProperties();
        this.getInstallLocation();
        String installPath = System.getProperty("osgi.install.area");
        if (installPath != null) {
            final URL url = new URL(installPath);
            final File f = new File(url.getFile());
            installPath = f.getAbsolutePath();
            System.setProperty("com.jiuqi.dna.rootpath", installPath);
            System.getProperties().put("osgi.configuration.area", String.valueOf(installPath) + "/work");
        }
        this.processConfiguration2();
        if (args.length > 0 && "stop".equals(args[0])) {
            ServerDaemon.stopServer();
        }
        else if (args.length > 0 && "restart".equals(args[0])) {
            ServerDaemon.restartServer();
        }
        else {
            ServerDaemon.startServer();
        }
        if (Boolean.getBoolean("out.redirect")) {
            IORedirecter.redirectIO(installPath);
        }
        if (Boolean.getBoolean("osgi.console.blockOnReady")) {
            System.setProperty("osgi.console.blockOnReady", "");
        }
        final URL[] bootPath = this.getBootPath(this.bootLocation);
        this.setupJNI(bootPath);
        if (!this.checkVersion(System.getProperty("java.version"), System.getProperty("osgi.requiredJavaVersion"))) {
            return;
        }
        this.setSecurityPolicy(bootPath);
        this.handleSplash(bootPath);
        this.beforeFwkInvocation();
        this.invokeFramework(passThruArgs, bootPath);
    }
    
    protected void beforeFwkInvocation() {
    }
    
    private final void processConfiguration2() {
        System.setProperty("osgi.clean", "true");
        System.setProperty("org.eclipse.equinox.http.jetty.autostart", "false");
        System.setProperty("POOL_MAX", "100000");
        System.setProperty("osgi.noShutdown", "true");
        final StringBuffer initBundles = new StringBuffer();
        final String serverRoot = System.getProperty("com.jiuqi.dna.rootpath");
        final File f = new File(serverRoot);
        String serverRootPath = StringUtil.formatPath(f.getPath(), false);
        if (serverRootPath.endsWith("/")) {
            serverRootPath = serverRootPath.substring(0, serverRootPath.length() - 1);
        }
        String[] bundleDirs;
        for (int length = (bundleDirs = Main.bundleDirs).length, i = 0; i < length; ++i) {
            final String bundleDir = bundleDirs[i];
            final File fBundleDir = new File(serverRoot, bundleDir);
            this.doStruct(fBundleDir, initBundles, serverRootPath);
        }
        if (Boolean.getBoolean("launcher.debug")) {
            System.out.println(initBundles.toString());
        }
        System.setProperty("osgi.bundles", initBundles.toString());
    }
    
    private final void doStruct(final File fBundleDir, final StringBuffer configBuffer, final String serverRootPath) {
        if (BundleUtils.isBundle(fBundleDir)) {
            final String bundleId = BundleUtils.getBundleId(fBundleDir);
            if (this.isNotLoadBundles(bundleId)) {
                return;
            }
            String filePath = StringUtil.formatPath(fBundleDir.getPath(), false);
            filePath = filePath.substring(serverRootPath.length());
            if (this.canStartBundle(fBundleDir)) {
                filePath = "reference:file:.." + filePath + "@start";
            }
            else {
                filePath = "reference:file:.." + filePath;
            }
            configBuffer.append(filePath);
            configBuffer.append(",");
        }
        else if (fBundleDir.isDirectory()) {
            File[] listFiles;
            for (int length = (listFiles = fBundleDir.listFiles()).length, i = 0; i < length; ++i) {
                final File f = listFiles[i];
                this.doStruct(f, configBuffer, serverRootPath);
            }
        }
    }
    
    private final boolean isNotLoadBundles(final String bundleId) {
        return ArrayUtils.contains((Object[])Main.notLoadBundles, (Object)bundleId);
    }
    
    private boolean canStartBundle(final File bundleFile) {
        return !BundleUtils.isFragmentBundle(bundleFile);
    }
    
    protected void setSecurityPolicy(final URL[] bootPath) {
        String eclipseSecurity = System.getProperty("eclipse.security");
        if (eclipseSecurity != null) {
            SecurityManager sm = System.getSecurityManager();
            boolean setSM = false;
            if (sm == null) {
                if (eclipseSecurity.length() < 1) {
                    eclipseSecurity = "java.lang.SecurityManager";
                }
                try {
                    final Class clazz = Class.forName(eclipseSecurity);
                    sm = clazz.newInstance();
                    setSM = true;
                }
                catch (final Throwable t) {
                    System.getProperties().put("java.security.manager", eclipseSecurity);
                }
            }
            final ProtectionDomain domain = Main.class.getProtectionDomain();
            CodeSource source = null;
            if (domain != null) {
                source = Main.class.getProtectionDomain().getCodeSource();
            }
            if (domain == null || source == null) {
                this.log("Can not automatically set the security manager. Please use a policy file.");
                return;
            }
            final URL[] rootURLs = new URL[bootPath.length + 1];
            rootURLs[0] = source.getLocation();
            System.arraycopy(bootPath, 0, rootURLs, 1, bootPath.length);
            final Policy eclipsePolicy = new EclipsePolicy(Policy.getPolicy(), rootURLs);
            Policy.setPolicy(eclipsePolicy);
            if (setSM) {
                System.setSecurityManager(sm);
            }
        }
    }
    
    private void invokeFramework(final String[] passThruArgs, final URL[] bootPath) throws ClassNotFoundException, NoSuchMethodException, IllegalAccessException, Error, Exception, InvocationTargetException {
        final String type = System.getProperty("osgi.frameworkParentClassloader", System.getProperty("osgi.parentClassloader", "boot"));
        ClassLoader parent = null;
        if ("app".equalsIgnoreCase(type)) {
            parent = ClassLoader.getSystemClassLoader();
        }
        else if ("ext".equalsIgnoreCase(type)) {
            final ClassLoader appCL = ClassLoader.getSystemClassLoader();
            if (appCL != null) {
                parent = appCL.getParent();
            }
        }
        else if ("current".equalsIgnoreCase(type)) {
            parent = this.getClass().getClassLoader();
        }
        final URLClassLoader loader = new StartupClassLoader(bootPath, parent);
        final Class clazz = loader.loadClass("org.eclipse.core.runtime.adaptor.EclipseStarter");
        final Method method = clazz.getDeclaredMethod("run", String[].class, Runnable.class);
        try {
            method.invoke(clazz, passThruArgs, this.splashHandler);
        }
        catch (final InvocationTargetException e) {
            if (e.getTargetException() instanceof Error) {
                throw (Error)e.getTargetException();
            }
            if (e.getTargetException() instanceof Exception) {
                throw (Exception)e.getTargetException();
            }
            throw e;
        }
    }
    
    private boolean checkVersion(final String availableVersion, final String requiredVersion) {
        if (requiredVersion == null || availableVersion == null) {
            return true;
        }
        try {
            final Identifier required = new Identifier(requiredVersion);
            final Identifier available = new Identifier(availableVersion);
            final boolean compatible = available.isGreaterEqualTo(required);
            if (!compatible) {
                System.getProperties().put("eclipse.exitcode", "14");
                System.getProperties().put("eclipse.exitdata", "<title>Incompatible JVM</title>Version " + availableVersion + " of the JVM is not suitable for this product. Version: " + requiredVersion + " or greater is required.");
            }
            return compatible;
        }
        catch (final SecurityException ex) {
            return true;
        }
        catch (final NumberFormatException ex2) {
            return true;
        }
    }
    
    protected String decode(String urlString) {
        try {
            final Class clazz = URLDecoder.class;
            final Method method = clazz.getDeclaredMethod("decode", String.class, String.class);
            if (urlString.indexOf(43) >= 0) {
                final int len = urlString.length();
                final StringBuffer buf = new StringBuffer(len);
                for (int i = 0; i < len; ++i) {
                    final char c = urlString.charAt(i);
                    if (c == '+') {
                        buf.append("%2B");
                    }
                    else {
                        buf.append(c);
                    }
                }
                urlString = buf.toString();
            }
            final Object result = method.invoke(null, urlString, "UTF-8");
            if (result != null) {
                return (String)result;
            }
        }
        catch (final Exception ex) {}
        boolean replaced = false;
        final byte[] encodedBytes = urlString.getBytes();
        final int encodedLength = encodedBytes.length;
        final byte[] decodedBytes = new byte[encodedLength];
        int decodedLength = 0;
        for (int j = 0; j < encodedLength; ++j) {
            byte b = encodedBytes[j];
            if (b == 37) {
                if (j + 2 >= encodedLength) {
                    throw new IllegalArgumentException("Malformed URL (\"" + urlString + "\"): % must be followed by 2 digits.");
                }
                final byte enc1 = encodedBytes[++j];
                final byte enc2 = encodedBytes[++j];
                b = (byte)((this.hexToByte(enc1) << 4) + this.hexToByte(enc2));
                replaced = true;
            }
            decodedBytes[decodedLength++] = b;
        }
        if (!replaced) {
            return urlString;
        }
        try {
            return new String(decodedBytes, 0, decodedLength, "UTF-8");
        }
        catch (final UnsupportedEncodingException ex2) {
            return new String(decodedBytes, 0, decodedLength);
        }
    }
    
    protected String[] getArrayFromList(final String prop) {
        if (prop == null || prop.trim().equals("")) {
            return new String[0];
        }
        final Vector list = new Vector();
        final StringTokenizer tokens = new StringTokenizer(prop, ",");
        while (tokens.hasMoreTokens()) {
            final String token = tokens.nextToken().trim();
            if (!token.equals("")) {
                list.addElement(token);
            }
        }
        return list.isEmpty() ? new String[0] : list.toArray(new String[list.size()]);
    }
    
    private URL[] getDevPath(final URL base) throws IOException {
        final ArrayList result = new ArrayList(5);
        if (this.inDevelopmentMode) {
            this.addDevEntries(base, result, "org.eclipse.osgi");
        }
        this.addBaseJars(base, result);
        return result.toArray(new URL[result.size()]);
    }
    
    URL constructURL(final URL url, final String name) {
        final String externalForm = url.toExternalForm();
        if (externalForm.endsWith(".jar")) {
            try {
                return new URL("jar:" + url + "!/" + name);
            }
            catch (final MalformedURLException ex) {}
        }
        try {
            return new URL(url, name);
        }
        catch (final MalformedURLException ex2) {
            return null;
        }
    }
    
    private void readFrameworkExtensions(final URL base, final ArrayList result) throws IOException {
        final String[] extensions = this.getArrayFromList(System.getProperties().getProperty("osgi.framework.extensions"));
        final String parent = new File(base.getFile()).getParent().toString();
        final ArrayList extensionResults = new ArrayList(extensions.length);
        for (int i = 0; i < extensions.length; ++i) {
            final String path = this.searchFor(extensions[i], parent);
            if (path == null) {
                this.log("Could not find extension: " + extensions[i]);
            }
            else {
                if (this.debug) {
                    System.out.println("Loading extension: " + extensions[i]);
                }
                URL extensionURL = null;
                if (this.installLocation.getProtocol().equals("file")) {
                    extensionResults.add(path);
                    extensionURL = new File(path).toURL();
                }
                else {
                    extensionURL = new URL(this.installLocation.getProtocol(), this.installLocation.getHost(), this.installLocation.getPort(), path);
                }
                Properties extensionProperties = null;
                try {
                    extensionProperties = this.loadProperties(this.constructURL(extensionURL, "eclipse.properties"));
                }
                catch (final IOException ex) {
                    if (this.debug) {
                        System.out.println("\teclipse.properties not found");
                    }
                }
                String extensionClassPath = null;
                if (extensionProperties != null) {
                    extensionClassPath = extensionProperties.getProperty("osgi.frameworkClassPath");
                }
                else {
                    extensionProperties = new Properties();
                }
                final String[] entries = (extensionClassPath == null || extensionClassPath.length() == 0) ? new String[] { "" } : this.getArrayFromList(extensionClassPath);
                String qualifiedPath;
                if (System.getProperty("osgi.frameworkClassPath") == null) {
                    qualifiedPath = ".";
                }
                else {
                    qualifiedPath = "";
                }
                for (int j = 0; j < entries.length; ++j) {
                    qualifiedPath = String.valueOf(qualifiedPath) + ", file:" + path + entries[j];
                }
                extensionProperties.put("osgi.frameworkClassPath", qualifiedPath);
                this.mergeProperties(System.getProperties(), extensionProperties);
                if (this.inDevelopmentMode) {
                    this.addDevEntries(extensionURL, result, extensions[i]);
                }
            }
        }
        this.extensionPaths = extensionResults.toArray(new String[extensionResults.size()]);
    }
    
    private void addBaseJars(final URL base, final ArrayList result) throws IOException {
        String baseJarList = System.getProperty("osgi.frameworkClassPath");
        if (baseJarList == null) {
            this.readFrameworkExtensions(base, result);
            baseJarList = System.getProperties().getProperty("osgi.frameworkClassPath");
        }
        final File fwkFile = new File(base.getFile());
        final boolean fwkIsDirectory = fwkFile.isDirectory();
        if (fwkIsDirectory) {
            System.getProperties().put("osgi.framework.shape", "folder");
        }
        else {
            System.getProperties().put("osgi.framework.shape", "jar");
        }
        String fwkPath = new File(new File(base.getFile()).getParent()).getAbsolutePath();
        if (Character.isUpperCase(fwkPath.charAt(0))) {
            final char[] chars = fwkPath.toCharArray();
            chars[0] = Character.toLowerCase(chars[0]);
            fwkPath = new String(chars);
        }
        System.getProperties().put("osgi.syspath", fwkPath);
        final String[] baseJars = this.getArrayFromList(baseJarList);
        if (baseJars.length != 0) {
            for (int i = 0; i < baseJars.length; ++i) {
                final String string = baseJars[i];
                try {
                    if (string.equals(".")) {
                        this.addEntry(base, result);
                    }
                    URL url = null;
                    if (string.startsWith("file:")) {
                        url = new File(string.substring(5)).toURL();
                    }
                    else {
                        url = new URL(string);
                    }
                    this.addEntry(url, result);
                }
                catch (final MalformedURLException ex) {
                    this.addEntry(new URL(base, string), result);
                }
            }
            return;
        }
        if (!this.inDevelopmentMode && new File(base.getFile()).isDirectory()) {
            throw new IOException("Unable to initialize osgi.frameworkClassPath");
        }
        this.addEntry(base, result);
    }
    
    protected void addEntry(final URL url, final List result) {
        if (new File(url.getFile()).exists()) {
            result.add(url);
        }
    }
    
    private void addDevEntries(final URL base, final List result, final String symbolicName) throws MalformedURLException {
        if (this.devClassPathProps == null) {
            return;
        }
        String devPathList = this.devClassPathProps.getProperty(symbolicName);
        if (devPathList == null) {
            devPathList = this.devClassPathProps.getProperty("*");
        }
        final String[] locations = this.getArrayFromList(devPathList);
        for (int i = 0; i < locations.length; ++i) {
            final String location = locations[i];
            final File path = new File(location);
            URL url;
            if (path.isAbsolute()) {
                url = path.toURL();
            }
            else {
                final char lastChar = location.charAt(location.length() - 1);
                if (location.endsWith(".jar") || lastChar == '/' || lastChar == '\\') {
                    url = new URL(base, location);
                }
                else {
                    url = new URL(base, String.valueOf(location) + "/");
                }
            }
            this.addEntry(url, result);
        }
    }
    
    protected URL[] getBootPath(final String base) throws IOException {
        URL url = null;
        if (base != null) {
            url = buildURL(base, true);
        }
        else {
            url = this.getInstallLocation();
            String path = null;
            File dir = new File(url.getFile(), "bundles");
            if (dir.exists()) {
                path = dir.getAbsolutePath();
            }
            else {
                dir = new File(url.getFile(), "thr");
                if (dir.exists()) {
                    path = dir.getAbsolutePath();
                }
            }
            path = this.searchFor(this.framework, path);
            if (path == null) {
                throw new RuntimeException("Could not find framework");
            }
            if (url.getProtocol().equals("file")) {
                url = new File(path).toURL();
            }
            else {
                url = new URL(url.getProtocol(), url.getHost(), url.getPort(), path);
            }
        }
        if (System.getProperty("osgi.framework") == null) {
            System.getProperties().put("osgi.framework", url.toExternalForm());
        }
        if (this.debug) {
            System.out.println("Framework located:\n    " + url.toExternalForm());
        }
        final URL[] result = this.getDevPath(url);
        if (this.debug) {
            System.out.println("Framework classpath:");
            for (int i = 0; i < result.length; ++i) {
                System.out.println("    " + result[i].toExternalForm());
            }
        }
        return result;
    }
    
    protected String searchFor(final String target, final String start) {
        return this.searchFor(target, null, start);
    }
    
    protected String searchFor(final String target, final String targetSuffix, final String start) {
        final String[] candidates = new File(start).list();
        if (candidates == null) {
            return null;
        }
        final ArrayList matches = new ArrayList(2);
        for (int i = 0; i < candidates.length; ++i) {
            if (candidates[i].equals(target) || candidates[i].startsWith(String.valueOf(target) + "_")) {
                matches.add(candidates[i]);
            }
        }
        final String[] names = matches.toArray(new String[matches.size()]);
        final int result = this.findMax(names);
        if (result == -1) {
            return null;
        }
        final File candidate = new File(start, names[result]);
        return String.valueOf(candidate.getAbsolutePath().replace(File.separatorChar, '/')) + (candidate.isDirectory() ? "/" : "");
    }
    
    protected int findMax(final String[] candidates) {
        int result = -1;
        Object maxVersion = null;
        for (int i = 0; i < candidates.length; ++i) {
            final String name = candidates[i];
            String version = "";
            final int index = name.indexOf(95);
            if (index != -1) {
                version = name.substring(index + 1);
            }
            final Object currentVersion = this.getVersionElements(version);
            if (maxVersion == null) {
                result = i;
                maxVersion = currentVersion;
            }
            else if (this.compareVersion((Object[])maxVersion, (Object[])currentVersion) < 0) {
                result = i;
                maxVersion = currentVersion;
            }
        }
        return result;
    }
    
    private int compareVersion(final Object[] left, final Object[] right) {
        int result = ((Integer)left[0]).compareTo((Integer)right[0]);
        if (result != 0) {
            return result;
        }
        result = ((Integer)left[1]).compareTo((Integer)right[1]);
        if (result != 0) {
            return result;
        }
        result = ((Integer)left[2]).compareTo((Integer)right[2]);
        if (result != 0) {
            return result;
        }
        return ((String)left[3]).compareTo((String)right[3]);
    }
    
    private Object[] getVersionElements(String version) {
        if (version.endsWith(".jar")) {
            version = version.substring(0, version.length() - 4);
        }
        final Object[] result = { new Integer(0), new Integer(0), new Integer(0), "" };
        final StringTokenizer t = new StringTokenizer(version, ".");
        int i = 0;
        while (t.hasMoreTokens() && i < 4) {
            final String token = t.nextToken();
            if (i < 3) {
                try {
                    result[i++] = new Integer(token);
                    continue;
                }
                catch (final Exception ex) {
                    break;
                }
            }
            result[i++] = token;
        }
        return result;
    }
    
    private static URL buildURL(final String spec, final boolean trailingSlash) {
        if (spec == null) {
            return null;
        }
        final boolean isFile = spec.startsWith("file:");
        try {
            if (!isFile) {
                return new URL(spec);
            }
            final File toAdjust = new File(spec.substring(5));
            if (toAdjust.isDirectory()) {
                return adjustTrailingSlash(toAdjust.toURL(), trailingSlash);
            }
            return toAdjust.toURL();
        }
        catch (final MalformedURLException ex) {
            if (isFile) {
                return null;
            }
            try {
                final File toAdjust = new File(spec);
                if (toAdjust.isDirectory()) {
                    return adjustTrailingSlash(toAdjust.toURL(), trailingSlash);
                }
                return toAdjust.toURL();
            }
            catch (final MalformedURLException ex2) {
                return null;
            }
        }
    }
    
    private static URL adjustTrailingSlash(final URL url, final boolean trailingSlash) throws MalformedURLException {
        String file = url.getFile();
        if (trailingSlash == file.endsWith("/")) {
            return url;
        }
        file = (trailingSlash ? (String.valueOf(file) + "/") : file.substring(0, file.length() - 1));
        return new URL(url.getProtocol(), url.getHost(), file);
    }
    
    private URL buildLocation(final String property, final URL defaultLocation, final String userDefaultAppendage) {
        URL result = null;
        String location = System.getProperty(property);
        System.getProperties().remove(property);
        try {
            if (location == null) {
                result = defaultLocation;
            }
            else {
                if (location.equalsIgnoreCase("@none")) {
                    return null;
                }
                if (location.equalsIgnoreCase("@noDefault")) {
                    result = buildURL(location, true);
                }
                else {
                    if (location.startsWith("@user.home")) {
                        final String base = this.substituteVar(location, "@user.home", "user.home");
                        location = new File(base, userDefaultAppendage).getAbsolutePath();
                    }
                    else if (location.startsWith("@user.dir")) {
                        final String base = this.substituteVar(location, "@user.dir", "user.dir");
                        location = new File(base, userDefaultAppendage).getAbsolutePath();
                    }
                    result = buildURL(location, true);
                }
            }
        }
        finally {
            if (result != null) {
                System.getProperties().put(property, result.toExternalForm());
            }
        }
        if (result != null) {
            System.getProperties().put(property, result.toExternalForm());
        }
        return result;
    }
    
    private String substituteVar(final String source, final String var, final String prop) {
        final String value = System.getProperty(prop, "");
        return String.valueOf(value) + source.substring(var.length());
    }
    
    private String computeDefaultConfigurationLocation() {
        final URL install = this.getInstallLocation();
        if (install.getProtocol().equals("file")) {
            final File installDir = new File(install.getFile());
            if (canWrite(installDir)) {
                return String.valueOf(installDir.getAbsolutePath()) + File.separator + "configuration/";
            }
        }
        return this.computeDefaultUserAreaLocation("configuration/");
    }
    
    private static boolean canWrite(final File installDir) {
        if (!installDir.canWrite()) {
            return false;
        }
        if (!installDir.isDirectory()) {
            return false;
        }
        File fileTest = null;
        try {
            fileTest = File.createTempFile("writtableArea", ".dll", installDir);
        }
        catch (final IOException ex) {
            return false;
        }
        finally {
            if (fileTest != null) {
                fileTest.delete();
            }
        }
        if (fileTest != null) {
            fileTest.delete();
        }
        return true;
    }
    
    private String computeDefaultUserAreaLocation(final String pathAppendage) {
        final URL installURL = this.getInstallLocation();
        if (installURL == null) {
            return null;
        }
        final File installDir = new File(installURL.getFile());
        int hashCode;
        try {
            hashCode = installDir.getCanonicalPath().hashCode();
        }
        catch (final IOException ex) {
            hashCode = installDir.getAbsolutePath().hashCode();
        }
        if (hashCode < 0) {
            hashCode = -hashCode;
        }
        final String installDirHash = String.valueOf(hashCode);
        String appName = ".eclipse";
        final File eclipseProduct = new File(installDir, ".eclipseproduct");
        if (eclipseProduct.exists()) {
            final Properties props = new Properties();
            try {
                props.load(new FileInputStream(eclipseProduct));
                String appId = props.getProperty("id");
                if (appId == null || appId.trim().length() == 0) {
                    appId = "eclipse";
                }
                String appVersion = props.getProperty("version");
                if (appVersion == null || appVersion.trim().length() == 0) {
                    appVersion = "";
                }
                appName = String.valueOf(appName) + File.separator + appId + "_" + appVersion + "_" + installDirHash;
            }
            catch (final IOException ex2) {
                appName = String.valueOf(appName) + File.separator + installDirHash;
            }
        }
        else {
            appName = String.valueOf(appName) + File.separator + installDirHash;
        }
        final String userHome = System.getProperty("user.home");
        return new File(userHome, String.valueOf(appName) + "/" + pathAppendage).getAbsolutePath();
    }
    
    public static void main(final String argString) {
        final Vector list = new Vector(5);
        final StringTokenizer tokens = new StringTokenizer(argString, " ");
        while (tokens.hasMoreElements()) {
            list.addElement(tokens.nextElement());
        }
        main(list.toArray(new String[list.size()]));
    }
    
    public static void main(final String[] args) {
        int result = 0;
        try {
            result = new Main().run(args);
        }
        catch (final Throwable t) {
            t.printStackTrace();
        }
    }
    
    public int run(final String[] args) {
        int result = 0;
        Label_0220: {
            try {
                this.basicRun(args);
                final String exitCode = System.getProperty("eclipse.exitcode");
                try {
                    result = ((exitCode == null) ? 0 : Integer.parseInt(exitCode));
                }
                catch (final NumberFormatException ex) {
                    result = 17;
                }
            }
            catch (final Throwable e) {
                if (!"13".equals(System.getProperty("eclipse.exitcode"))) {
                    this.log("Exception launching the Eclipse Platform:");
                    this.log(e);
                    String message = "An error has occurred";
                    if (this.logFile == null) {
                        message = String.valueOf(message) + " and could not be logged: \n" + e.getMessage();
                    }
                    else {
                        message = String.valueOf(message) + ".  See the log file\n" + this.logFile.getAbsolutePath();
                    }
                    System.getProperties().put("eclipse.exitdata", message);
                }
                result = 13;
                break Label_0220;
            }
            finally {
                this.takeDownSplash();
                if (this.bridge != null) {
                    this.bridge.uninitialize();
                }
            }
            this.takeDownSplash();
            if (this.bridge != null) {
                this.bridge.uninitialize();
            }
        }
        System.getProperties().put("eclipse.exitcode", Integer.toString(result));
        this.setExitData();
        return result;
    }
    
    private void setExitData() {
        final String data = System.getProperty("eclipse.exitdata");
        if (data == null || this.bridge == null) {
            return;
        }
        this.bridge.setExitData(this.exitData, data);
    }
    
    protected String[] processCommandLine(final String[] args) {
        if (args.length == 0) {
            return args;
        }
        final int[] configArgs = new int[args.length];
        configArgs[0] = -1;
        int configArgIndex = 0;
        for (int i = 0; i < args.length; ++i) {
            boolean found = false;
            if (args[i].equalsIgnoreCase("-debug")) {
                this.debug = true;
            }
            else {
                if (args[i].equalsIgnoreCase("-nosplash")) {
                    this.splashDown = true;
                    found = true;
                }
                if (args[i].equalsIgnoreCase("-noExit")) {
                    System.getProperties().put("osgi.noShutdown", "true");
                    found = true;
                }
                if (args[i].equalsIgnoreCase("-initialize")) {
                    this.initialize = true;
                }
                else if (args[i].equalsIgnoreCase("-dev") && (i + 1 == args.length || (i + 1 < args.length && args[i + 1].startsWith("-")))) {
                    this.inDevelopmentMode = true;
                }
                else {
                    if (args[i].equalsIgnoreCase("-showsplash")) {
                        this.showSplash = true;
                        found = true;
                        if (i + 1 < args.length && !args[i + 1].startsWith("-")) {
                            configArgs[configArgIndex++] = i++;
                            this.splashLocation = args[i];
                        }
                    }
                    if (found) {
                        configArgs[configArgIndex++] = i;
                    }
                    else if (args[i].equalsIgnoreCase("-vmargs")) {
                        args[i] = null;
                        ++i;
                        this.vmargs = new String[args.length - i];
                        int j = 0;
                        while (i < args.length) {
                            this.vmargs[j++] = args[i];
                            args[i] = null;
                            ++i;
                        }
                    }
                    else if (i != args.length - 1) {
                        if (!args[i + 1].startsWith("-")) {
                            final String arg = args[++i];
                            if (args[i - 1].equalsIgnoreCase("-dev")) {
                                this.inDevelopmentMode = true;
                                this.devClassPathProps = this.processDevArg(arg);
                                if (this.devClassPathProps != null) {
                                    this.devClassPath = this.devClassPathProps.getProperty("org.eclipse.osgi");
                                    if (this.devClassPath == null) {
                                        this.devClassPath = this.devClassPathProps.getProperty("*");
                                    }
                                }
                            }
                            else {
                                if (args[i - 1].equalsIgnoreCase("-framework")) {
                                    this.framework = arg;
                                    found = true;
                                }
                                if (args[i - 1].equalsIgnoreCase("-os")) {
                                    this.os = arg;
                                    System.getProperties().put("os.type", this.os);
                                }
                                else if (args[i - 1].equalsIgnoreCase("-ws")) {
                                    this.ws = arg;
                                }
                                else if (args[i - 1].equalsIgnoreCase("-arch")) {
                                    this.arch = arg;
                                }
                                else {
                                    if (args[i - 1].equalsIgnoreCase("-install")) {
                                        System.getProperties().put("osgi.install.area", arg);
                                        found = true;
                                    }
                                    if (args[i - 1].equalsIgnoreCase("-configuration")) {
                                        System.getProperties().put("osgi.configuration.area", arg);
                                        found = true;
                                    }
                                    if (args[i - 1].equalsIgnoreCase("-exitdata")) {
                                        this.exitData = arg;
                                        found = true;
                                    }
                                    if (args[i - 1].equalsIgnoreCase("-name")) {
                                        found = true;
                                    }
                                    if (args[i - 1].equalsIgnoreCase("-startup")) {
                                        found = true;
                                    }
                                    if (args[i - 1].equalsIgnoreCase("-launcher")) {
                                        found = true;
                                    }
                                    if (args[i - 1].equalsIgnoreCase("--launcher.library")) {
                                        this.library = arg;
                                        found = true;
                                    }
                                    if (args[i - 1].equalsIgnoreCase("-endsplash")) {
                                        this.endSplash = arg;
                                        found = true;
                                    }
                                    if (args[i - 1].equalsIgnoreCase("-vm")) {
                                        this.vm = arg;
                                        found = true;
                                    }
                                    if (args[i - 1].equalsIgnoreCase("-nl")) {
                                        System.getProperties().put("osgi.nl", arg);
                                        found = true;
                                    }
                                    if (found) {
                                        configArgs[configArgIndex++] = i - 1;
                                        configArgs[configArgIndex++] = i;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        final String[] passThruArgs = new String[args.length - configArgIndex - ((this.vmargs == null) ? 0 : (this.vmargs.length + 1))];
        configArgIndex = 0;
        int k = 0;
        for (int l = 0; l < args.length; ++l) {
            if (l == configArgs[configArgIndex]) {
                ++configArgIndex;
            }
            else if (args[l] != null) {
                passThruArgs[k++] = args[l];
            }
        }
        return passThruArgs;
    }
    
    private Properties processDevArg(final String arg) {
        if (arg == null) {
            return null;
        }
        try {
            final URL location = new URL(arg);
            return this.load(location, null);
        }
        catch (final MalformedURLException ex) {
            final Properties result = new Properties();
            result.put("*", arg);
            return result;
        }
        catch (final IOException ex2) {
            return null;
        }
    }
    
    private URL getConfigurationLocation() {
        if (this.configurationLocation != null) {
            return this.configurationLocation;
        }
        this.configurationLocation = this.buildLocation("osgi.configuration.area", null, "");
        if (this.configurationLocation == null) {
            this.configurationLocation = this.buildLocation("osgi.configuration.area.default", null, "");
            if (this.configurationLocation == null) {
                this.configurationLocation = buildURL(this.computeDefaultConfigurationLocation(), true);
            }
        }
        if (this.configurationLocation != null) {
            System.getProperties().put("osgi.configuration.area", this.configurationLocation.toExternalForm());
        }
        if (this.debug) {
            System.out.println("Configuration location:\n    " + this.configurationLocation);
        }
        return this.configurationLocation;
    }
    
    private void processConfiguration() {
        URL baseConfigurationLocation = null;
        Properties baseConfiguration = null;
        if (System.getProperty("osgi.configuration.area") == null) {
            final String baseLocation = System.getProperty("osgi.baseConfiguration.area");
            if (baseLocation != null) {
                baseConfigurationLocation = buildURL(baseLocation, true);
            }
            if (baseConfigurationLocation == null) {
                try {
                    baseConfigurationLocation = new URL(this.getInstallLocation(), "configuration/");
                }
                catch (final MalformedURLException ex) {}
            }
            baseConfiguration = this.loadConfiguration(baseConfigurationLocation);
            if (baseConfiguration != null) {
                String location = baseConfiguration.getProperty("osgi.configuration.area");
                if (location != null) {
                    System.getProperties().put("osgi.configuration.area", location);
                }
                location = baseConfiguration.getProperty("osgi.install.area");
                if (location != null && System.getProperty("osgi.install.area") == null) {
                    System.getProperties().put("osgi.install.area", location);
                }
            }
        }
        Properties configuration = baseConfiguration;
        if (configuration == null || !this.getConfigurationLocation().equals(baseConfigurationLocation)) {
            configuration = this.loadConfiguration(this.getConfigurationLocation());
        }
        this.mergeProperties(System.getProperties(), configuration);
        if ("false".equalsIgnoreCase(System.getProperty("osgi.configuration.cascaded"))) {
            System.getProperties().remove("osgi.sharedConfiguration.area");
        }
        else {
            this.ensureAbsolute("osgi.sharedConfiguration.area");
            URL sharedConfigURL = this.buildLocation("osgi.sharedConfiguration.area", null, "");
            if (sharedConfigURL == null) {
                try {
                    sharedConfigURL = new URL(this.getInstallLocation(), "configuration/");
                }
                catch (final MalformedURLException ex2) {}
            }
            if (sharedConfigURL != null) {
                if (sharedConfigURL.equals(this.getConfigurationLocation())) {
                    System.getProperties().remove("osgi.sharedConfiguration.area");
                }
                else {
                    configuration = baseConfiguration;
                    if (!sharedConfigURL.equals(baseConfigurationLocation)) {
                        configuration = this.loadConfiguration(sharedConfigURL);
                    }
                    this.mergeProperties(System.getProperties(), configuration);
                    System.getProperties().put("osgi.sharedConfiguration.area", sharedConfigURL.toExternalForm());
                    if (this.debug) {
                        System.out.println("Shared configuration location:\n    " + sharedConfigURL.toExternalForm());
                    }
                }
            }
        }
        final String urlString = System.getProperty("osgi.framework", null);
        if (urlString != null) {
            final URL url = buildURL(urlString, true);
            System.getProperties().put("osgi.framework", url.toExternalForm());
            this.bootLocation = this.resolve(urlString);
        }
    }
    
    private void ensureAbsolute(final String locationProperty) {
        final String propertyValue = System.getProperty(locationProperty);
        if (propertyValue == null) {
            return;
        }
        URL locationURL = null;
        try {
            locationURL = new URL(propertyValue);
        }
        catch (final MalformedURLException ex) {
            return;
        }
        final String locationPath = locationURL.getPath();
        if (locationPath.startsWith("/")) {
            return;
        }
        final URL installURL = this.getInstallLocation();
        if (!locationURL.getProtocol().equals(installURL.getProtocol())) {
            return;
        }
        try {
            final URL absoluteURL = new URL(installURL, locationPath);
            System.getProperties().put(locationProperty, absoluteURL.toExternalForm());
        }
        catch (final MalformedURLException ex2) {}
    }
    
    private URL getInstallLocation() {
        if (this.installLocation != null) {
            return this.installLocation;
        }
        final String installArea = System.getProperty("osgi.install.area");
        if (installArea != null) {
            this.installLocation = buildURL(installArea, true);
            if (this.installLocation == null) {
                throw new IllegalStateException("Install location is invalid: " + installArea);
            }
            System.getProperties().put("osgi.install.area", this.installLocation.toExternalForm());
            if (this.debug) {
                System.out.println("Install location:\n    " + this.installLocation);
            }
            return this.installLocation;
        }
        else {
            final ProtectionDomain domain = Main.class.getProtectionDomain();
            CodeSource source = null;
            URL result = null;
            if (domain != null) {
                source = domain.getCodeSource();
            }
            if (source == null || domain == null) {
                if (this.debug) {
                    System.out.println("CodeSource location is null. Defaulting the install location to file:startup.jar");
                }
                try {
                    result = new URL("file:startup.jar");
                }
                catch (final MalformedURLException ex) {}
            }
            if (source != null) {
                result = source.getLocation();
            }
            String path = this.decode(result.getFile());
            final File file = new File(path);
            path = file.toString().replace('\\', '/');
            if (File.separatorChar == '\\' && Character.isUpperCase(path.charAt(0))) {
                final char[] chars = path.toCharArray();
                chars[0] = Character.toLowerCase(chars[0]);
                path = new String(chars);
            }
            System.setProperty("com.jiuqi.dna.launcher.path", file.getAbsolutePath());
            if (path.toLowerCase().endsWith(".jar")) {
                path = path.substring(0, path.lastIndexOf("/") + 1);
            }
            if (path.toLowerCase().endsWith("/bundles/")) {
                path = path.substring(0, path.length() - "/bundles/".length());
            }
            else if (path.toLowerCase().endsWith("/thr/")) {
                path = path.substring(0, path.length() - "/thr/".length());
            }
            while (true) {
                if (path.contains("#") || path.contains("%") || path.contains("!")) {
                    System.err.println("Install path: " + path);
                    System.err.println("ERROR: Install path contains illegal characters like \"#%!\".");
                    System.exit(1);
                    try {
                        try {
                            path = new File(path).toURL().getFile();
                        }
                        catch (final MalformedURLException ex2) {}
                        this.installLocation = new URL(result.getProtocol(), result.getHost(), result.getPort(), path);
                        System.getProperties().put("osgi.install.area", this.installLocation.toExternalForm());
                    }
                    catch (final MalformedURLException ex3) {}
                    if (this.debug) {
                        System.out.println("Install location:\n    " + this.installLocation);
                    }
                    return this.installLocation;
                }
                continue;
            }
        }
    }
    
    private Properties loadConfiguration(URL url) {
        Properties result = null;
        try {
            url = new URL(url, "config.ini");
        }
        catch (final MalformedURLException ex) {
            return null;
        }
        try {
            if (this.debug) {
                System.out.print("Configuration file:\n    " + url.toString());
            }
            result = this.loadProperties(url);
            if (this.debug) {
                System.out.println(" loaded");
            }
        }
        catch (final IOException ex2) {
            if (this.debug) {
                System.out.println(" not found or not read");
            }
        }
        return result;
    }
    
    private Properties loadProperties(final URL url) throws IOException {
        if (url == null) {
            return null;
        }
        Properties result = null;
        IOException originalException = null;
        try {
            result = this.load(url, null);
        }
        catch (final IOException e1) {
            originalException = e1;
            try {
                result = this.load(url, ".tmp");
            }
            catch (final IOException ex) {
                try {
                    result = this.load(url, ".bak");
                }
                catch (final IOException ex2) {
                    throw originalException;
                }
            }
        }
        return result;
    }
    
    private Properties load(URL url, final String suffix) throws IOException {
        if (suffix != null && !suffix.equals("")) {
            url = new URL(url.getProtocol(), url.getHost(), url.getPort(), String.valueOf(url.getFile()) + suffix);
        }
        final Properties props = new Properties();
        InputStream is = null;
        try {
            is = url.openStream();
            props.load(is);
        }
        finally {
            if (is != null) {
                try {
                    is.close();
                }
                catch (final IOException ex) {}
            }
        }
        if (is != null) {
            try {
                is.close();
            }
            catch (final IOException ex2) {}
        }
        return props;
    }
    
    private void handleSplash(final URL[] defaultPath) {
        if (this.initialize || this.splashDown || this.bridge == null) {
            this.showSplash = false;
            this.endSplash = null;
            return;
        }
        Label_0060: {
            if (!this.showSplash) {
                if (this.endSplash == null) {
                    break Label_0060;
                }
            }
            try {
                Runtime.getRuntime().addShutdownHook(this.splashHandler);
            }
            catch (final Throwable t) {}
        }
        if (this.endSplash != null) {
            this.showSplash = false;
            return;
        }
        if (!this.showSplash) {
            return;
        }
        this.splashLocation = this.getSplashLocation(defaultPath);
        if (this.debug) {
            System.out.println("Splash location:\n    " + this.splashLocation);
        }
        if (this.splashLocation == null) {
            return;
        }
        this.bridge.showSplash(this.splashLocation);
        final long handle = this.bridge.getSplashHandle();
        if (handle != 0L && handle != -1L) {
            System.getProperties().put("org.eclipse.equinox.launcher.splash.handle", String.valueOf(handle));
            System.getProperties().put("org.eclipse.equinox.launcher.splash.location", this.splashLocation);
            this.bridge.updateSplash();
        }
    }
    
    protected void takeDownSplash() {
        if (this.splashDown || this.bridge == null) {
            return;
        }
        this.splashDown = this.bridge.takeDownSplash();
        System.getProperties().remove("org.eclipse.equinox.launcher.splash.handle");
        try {
            Runtime.getRuntime().removeShutdownHook(this.splashHandler);
        }
        catch (final Throwable t) {}
    }
    
    private String getSplashLocation(final URL[] bootPath) {
        if (this.splashLocation != null && !Character.isDigit(this.splashLocation.charAt(0)) && new File(this.splashLocation).exists()) {
            System.getProperties().put("osgi.splashLocation", this.splashLocation);
            return this.splashLocation;
        }
        String result = System.getProperty("osgi.splashLocation");
        if (result != null) {
            return result;
        }
        final String splashPath = System.getProperty("osgi.splashPath");
        if (splashPath != null) {
            final String[] entries = this.getArrayFromList(splashPath);
            final ArrayList path = new ArrayList(entries.length);
            for (int i = 0; i < entries.length; ++i) {
                String entry = this.resolve(entries[i]);
                if (entry == null || entry.startsWith("file:")) {
                    final File entryFile = new File(entry.substring(5).replace('/', File.separatorChar));
                    entry = this.searchFor(entryFile.getName(), entryFile.getParent());
                    if (entry != null) {
                        path.add(entry);
                    }
                }
                else {
                    this.log("Invalid splash path entry: " + entries[i]);
                }
            }
            result = this.searchForSplash(path.toArray(new String[path.size()]));
            if (result != null) {
                System.getProperties().put("osgi.splashLocation", result);
                return result;
            }
        }
        result = this.searchSplash(bootPath, "bundles");
        if (result == null) {
            result = this.searchSplash(bootPath, "thr");
        }
        return result;
    }
    
    private String searchSplash(final URL[] bootPath, final String dir) {
        String result = null;
        String temp = bootPath[0].getFile();
        temp = temp.replace('/', File.separatorChar);
        final int ix = temp.lastIndexOf(String.valueOf(dir) + File.separator);
        if (ix != -1) {
            final int pix = temp.indexOf(File.separator, ix + 8);
            if (pix != -1) {
                temp = temp.substring(0, pix);
                result = this.searchForSplash(new String[] { temp });
                if (result != null) {
                    System.getProperties().put("osgi.splashLocation", result);
                }
            }
        }
        return result;
    }
    
    private String searchForSplash(final String[] searchPath) {
        if (searchPath == null) {
            return null;
        }
        String locale = (String)System.getProperties().get("osgi.nl");
        if (locale == null) {
            locale = Locale.getDefault().toString();
        }
        final String[] nlVariants = buildNLVariants(locale);
        for (int i = 0; i < nlVariants.length; ++i) {
            for (int j = 0; j < searchPath.length; ++j) {
                if (this.isJAR(searchPath[j])) {
                    final String result = this.extractSplashFromJAR(searchPath[j], nlVariants[i]);
                    if (result != null) {
                        return result;
                    }
                }
                else {
                    String path = searchPath[j];
                    if (!path.endsWith(File.separator)) {
                        path = String.valueOf(path) + File.separator;
                    }
                    path = String.valueOf(path) + nlVariants[i];
                    final File result2 = new File(path);
                    if (result2.exists()) {
                        return result2.getAbsolutePath();
                    }
                }
            }
        }
        return null;
    }
    
    private static void transferStreams(final InputStream source, final OutputStream destination) {
        final byte[] buffer = new byte[8096];
        try {
            while (true) {
                int bytesRead = -1;
                try {
                    bytesRead = source.read(buffer);
                }
                catch (final IOException ex) {
                    return;
                }
                if (bytesRead == -1) {
                    break;
                }
                try {
                    destination.write(buffer, 0, bytesRead);
                }
                catch (final IOException ex2) {
                    return;
                }
            }
        }
        finally {
            try {
                source.close();
            }
            catch (final IOException ex3) {
                try {
                    destination.close();
                }
                catch (final IOException ex4) {}
            }
            finally {
                try {
                    destination.close();
                }
                catch (final IOException ex5) {}
            }
            try {
                destination.close();
            }
            catch (final IOException ex6) {}
        }
        try {
            source.close();
        }
        catch (final IOException ex7) {
            try {
                destination.close();
            }
            catch (final IOException ex8) {}
        }
        finally {
            try {
                destination.close();
            }
            catch (final IOException ex9) {}
        }
        try {
            destination.close();
        }
        catch (final IOException ex10) {}
    }
    
    private String extractSplashFromJAR(final String jarPath, final String splashPath) {
        final String configLocation = System.getProperty("osgi.configuration.area");
        if (configLocation == null) {
            this.log("Configuration area not set yet. Unable to extract splash from JAR'd plug-in: " + jarPath);
            return null;
        }
        final URL configURL = buildURL(configLocation, false);
        if (configURL == null) {
            return null;
        }
        File splash = new File(configURL.getPath(), "com.jiuqi.dna.launcher");
        final File jarFile = new File(jarPath);
        String cache = jarFile.getName();
        if (cache.endsWith(".jar")) {
            cache = cache.substring(0, cache.length() - 4);
        }
        splash = new File(splash, cache);
        splash = new File(splash, splashPath);
        if (splash.exists()) {
            boolean clean = false;
            for (int i = 0; i < this.commands.length; ++i) {
                if ("-clean".equalsIgnoreCase(this.commands[i])) {
                    clean = true;
                    splash.delete();
                    break;
                }
            }
            if (!clean) {
                return splash.getAbsolutePath();
            }
        }
        ZipFile file;
        try {
            file = new ZipFile(jarPath);
        }
        catch (final IOException e) {
            this.log("Exception looking for splash  in JAR file: " + jarPath);
            this.log(e);
            return null;
        }
        final ZipEntry entry = file.getEntry(splashPath.replace(File.separatorChar, '/'));
        if (entry == null) {
            return null;
        }
        InputStream input = null;
        try {
            input = file.getInputStream(entry);
        }
        catch (final IOException e2) {
            this.log("Exception opening splash: " + entry.getName() + " in JAR file: " + jarPath);
            this.log(e2);
            return null;
        }
        new File(splash.getParent()).mkdirs();
        OutputStream output;
        try {
            output = new BufferedOutputStream(new FileOutputStream(splash));
        }
        catch (final FileNotFoundException ex) {
            try {
                input.close();
            }
            catch (final IOException ex2) {}
            return null;
        }
        transferStreams(input, output);
        return splash.exists() ? splash.getAbsolutePath() : null;
    }
    
    private boolean isJAR(final String path) {
        if (path.endsWith(File.separator)) {
            return false;
        }
        int index = path.lastIndexOf(46);
        return index != -1 && ++index < path.length() && "JAR".equalsIgnoreCase(path.substring(index));
    }
    
    private static String[] buildNLVariants(final String locale) {
        String nl = locale;
        final ArrayList result = new ArrayList(4);
        while (true) {
            result.add("nl" + File.separatorChar + nl.replace('_', File.separatorChar) + File.separatorChar + "splash.bmp");
            final int lastSeparator = nl.lastIndexOf(95);
            if (lastSeparator == -1) {
                break;
            }
            nl = nl.substring(0, lastSeparator);
        }
        result.add("splash.bmp");
        return result.toArray(new String[result.size()]);
    }
    
    private String resolve(String urlString) {
        if (urlString.startsWith("reference:")) {
            urlString = urlString.substring(10);
            System.getProperties().put("osgi.framework", urlString);
        }
        if (urlString.startsWith("platform:/base/")) {
            final String path = urlString.substring("platform:/base/".length());
            return this.getInstallLocation() + path;
        }
        return urlString;
    }
    
    protected synchronized void log(final Object obj) {
        if (obj == null) {
            return;
        }
        try {
            this.openLogFile();
            try {
                if (this.newSession) {
                    this.log.write("!SESSION");
                    this.log.write(32);
                    final String timestamp = new Date().toString();
                    this.log.write(timestamp);
                    this.log.write(32);
                    for (int i = "!SESSION".length() + timestamp.length(); i < 78; ++i) {
                        this.log.write(45);
                    }
                    this.log.newLine();
                    this.newSession = false;
                }
                this.write(obj);
            }
            finally {
                if (this.logFile == null) {
                    if (this.log != null) {
                        this.log.flush();
                    }
                }
                else {
                    this.closeLogFile();
                }
            }
            if (this.logFile == null) {
                if (this.log != null) {
                    this.log.flush();
                }
            }
            else {
                this.closeLogFile();
            }
        }
        catch (final Exception e) {
            System.err.println("An exception occurred while writing to the platform log:");
            e.printStackTrace(System.err);
            System.err.println("Logging to the console instead.");
            try {
                this.log = this.logForStream(System.err);
                this.write(obj);
                this.log.flush();
            }
            catch (final Exception e2) {
                System.err.println("An exception occurred while logging to the console:");
                e2.printStackTrace(System.err);
            }
            return;
        }
        finally {
            this.log = null;
        }
        this.log = null;
    }
    
    private void write(final Object obj) throws IOException {
        if (obj == null) {
            return;
        }
        if (obj instanceof Throwable) {
            this.log.write("!STACK");
            this.log.newLine();
            ((Throwable)obj).printStackTrace(new PrintWriter(this.log));
        }
        else {
            this.log.write("!ENTRY");
            this.log.write(32);
            this.log.write("com.jiuqi.dna.launcher");
            this.log.write(32);
            this.log.write(String.valueOf(4));
            this.log.write(32);
            this.log.write(String.valueOf(0));
            this.log.write(32);
            this.log.write(this.getDate(new Date()));
            this.log.newLine();
            this.log.write("!MESSAGE");
            this.log.write(32);
            this.log.write(String.valueOf(obj));
        }
        this.log.newLine();
    }
    
    protected String getDate(final Date date) {
        final Calendar c = Calendar.getInstance();
        c.setTime(date);
        final StringBuffer sb = new StringBuffer();
        this.appendPaddedInt(c.get(1), 4, sb).append('-');
        this.appendPaddedInt(c.get(2) + 1, 2, sb).append('-');
        this.appendPaddedInt(c.get(5), 2, sb).append(' ');
        this.appendPaddedInt(c.get(11), 2, sb).append(':');
        this.appendPaddedInt(c.get(12), 2, sb).append(':');
        this.appendPaddedInt(c.get(13), 2, sb).append('.');
        this.appendPaddedInt(c.get(14), 3, sb);
        return sb.toString();
    }
    
    private StringBuffer appendPaddedInt(final int value, int pad, final StringBuffer buffer) {
        if (--pad == 0) {
            return buffer.append(Integer.toString(value));
        }
        int padding = (int)Math.pow(10.0, pad);
        if (value >= padding) {
            return buffer.append(Integer.toString(value));
        }
        while (padding > value && padding > 1) {
            buffer.append('0');
            padding /= 10;
        }
        buffer.append(value);
        return buffer;
    }
    
    private void computeLogFileLocation() {
        final String logFileProp = System.getProperty("osgi.logfile");
        if (logFileProp != null) {
            if (this.logFile == null || !logFileProp.equals(this.logFile.getAbsolutePath())) {
                this.logFile = new File(logFileProp);
                new File(this.logFile.getParent()).mkdirs();
            }
            return;
        }
        final URL base = buildURL(System.getProperty("osgi.configuration.area"), false);
        if (base == null) {
            return;
        }
        this.logFile = new File(base.getPath(), String.valueOf(Long.toString(System.currentTimeMillis())) + ".log");
        new File(this.logFile.getParent()).mkdirs();
        System.getProperties().put("osgi.logfile", this.logFile.getAbsolutePath());
    }
    
    private int hexToByte(final byte b) {
        switch (b) {
            case 48: {
                return 0;
            }
            case 49: {
                return 1;
            }
            case 50: {
                return 2;
            }
            case 51: {
                return 3;
            }
            case 52: {
                return 4;
            }
            case 53: {
                return 5;
            }
            case 54: {
                return 6;
            }
            case 55: {
                return 7;
            }
            case 56: {
                return 8;
            }
            case 57: {
                return 9;
            }
            case 65:
            case 97: {
                return 10;
            }
            case 66:
            case 98: {
                return 11;
            }
            case 67:
            case 99: {
                return 12;
            }
            case 68:
            case 100: {
                return 13;
            }
            case 69:
            case 101: {
                return 14;
            }
            case 70:
            case 102: {
                return 15;
            }
            default: {
                throw new IllegalArgumentException("Switch error decoding URL");
            }
        }
    }
    
    private void openLogFile() throws IOException {
        this.computeLogFileLocation();
        try {
            this.log = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(this.logFile.getAbsolutePath(), true), "UTF-8"));
        }
        catch (final IOException e) {
            this.logFile = null;
            throw e;
        }
    }
    
    private BufferedWriter logForStream(final OutputStream output) {
        try {
            return new BufferedWriter(new OutputStreamWriter(output, "UTF-8"));
        }
        catch (final UnsupportedEncodingException ex) {
            return new BufferedWriter(new OutputStreamWriter(output));
        }
    }
    
    private void closeLogFile() throws IOException {
        try {
            if (this.log != null) {
                this.log.flush();
                this.log.close();
            }
        }
        finally {
            this.log = null;
        }
        this.log = null;
    }
    
    private void mergeProperties(final Properties destination, final Properties source) {
        if (destination == null || source == null) {
            return;
        }
        final Enumeration e = source.keys();
        while (e.hasMoreElements()) {
            final String key = e.nextElement();
            if (key.equals("osgi.frameworkClassPath")) {
                String destinationClasspath = destination.getProperty("osgi.frameworkClassPath");
                final String sourceClasspath = source.getProperty("osgi.frameworkClassPath");
                if (destinationClasspath == null) {
                    destinationClasspath = sourceClasspath;
                }
                else {
                    destinationClasspath = String.valueOf(destinationClasspath) + sourceClasspath;
                }
                destination.put("osgi.frameworkClassPath", destinationClasspath);
            }
            else {
                final String value = source.getProperty(key);
                if (destination.getProperty(key) != null) {
                    continue;
                }
                destination.put(key, value);
            }
        }
    }
    
    private void setupVMProperties() {
        if (this.vm != null) {
            System.getProperties().put("eclipse.vm", this.vm);
        }
        this.setMultiValueProperty("eclipse.vmargs", this.vmargs);
        this.setMultiValueProperty("eclipse.commands", this.commands);
    }
    
    private void setMultiValueProperty(final String property, final String[] value) {
        if (value != null) {
            final StringBuffer result = new StringBuffer(300);
            for (int i = 0; i < value.length; ++i) {
                if (value[i] != null) {
                    result.append(value[i]);
                    result.append('\n');
                }
            }
            System.getProperties().put(property, result.toString());
        }
    }
    
    public final class SplashHandler extends Thread
    {
        @Override
        public void run() {
            Main.this.takeDownSplash();
        }
        
        public void updateSplash() {
            if (Main.this.bridge != null) {
                Main.this.bridge.updateSplash();
            }
        }
    }
    
    static class Identifier
    {
        private static final String DELIM = ". _-";
        private int major;
        private int minor;
        private int service;
        
        Identifier(final int major, final int minor, final int service) {
            this.major = major;
            this.minor = minor;
            this.service = service;
        }
        
        Identifier(final String versionString) {
            final StringTokenizer tokenizer = new StringTokenizer(versionString, ". _-");
            if (tokenizer.hasMoreTokens()) {
                this.major = Integer.parseInt(tokenizer.nextToken());
            }
            if (tokenizer.hasMoreTokens()) {
                this.minor = Integer.parseInt(tokenizer.nextToken());
            }
            try {
                if (tokenizer.hasMoreTokens()) {
                    this.service = Integer.parseInt(tokenizer.nextToken());
                }
            }
            catch (final NumberFormatException ex) {}
        }
        
        boolean isGreaterEqualTo(final Identifier minimum) {
            return this.major >= minimum.major && (this.major > minimum.major || (this.minor >= minimum.minor && (this.minor > minimum.minor || this.service >= minimum.service)));
        }
    }
    
    private class EclipsePolicy extends Policy
    {
        private final Policy policy;
        private final URL[] urls;
        private final PermissionCollection allPermissions;
        Permission allPermission;
        
        EclipsePolicy(final Policy policy, final URL[] urls) {
            this.allPermission = new AllPermission();
            this.policy = policy;
            this.urls = urls;
            this.allPermissions = new PermissionCollection() {
                private static final long serialVersionUID = 3258131349494708277L;
                
                @Override
                public void add(final Permission permission) {
                }
                
                @Override
                public boolean implies(final Permission permission) {
                    return true;
                }
                
                @Override
                public Enumeration elements() {
                    return new Enumeration() {
                        int cur = 0;
                        
                        public boolean hasMoreElements() {
                            return this.cur < 1;
                        }
                        
                        public Object nextElement() {
                            if (this.cur == 0) {
                                this.cur = 1;
                                return EclipsePolicy.this.allPermission;
                            }
                            throw new NoSuchElementException();
                        }
                    };
                }
            };
        }
        
        @Override
        public PermissionCollection getPermissions(final CodeSource codesource) {
            if (this.contains(codesource.getLocation())) {
                return this.allPermissions;
            }
            return (this.policy == null) ? this.allPermissions : this.policy.getPermissions(codesource);
        }
        
        @Override
        public PermissionCollection getPermissions(final ProtectionDomain domain) {
            if (this.contains(domain.getCodeSource().getLocation())) {
                return this.allPermissions;
            }
            return (this.policy == null) ? this.allPermissions : this.policy.getPermissions(domain);
        }
        
        @Override
        public boolean implies(final ProtectionDomain domain, final Permission permission) {
            return this.contains(domain.getCodeSource().getLocation()) || this.policy == null || this.policy.implies(domain, permission);
        }
        
        @Override
        public void refresh() {
            if (this.policy != null) {
                this.policy.refresh();
            }
        }
        
        private boolean contains(final URL url) {
            for (int i = 0; i < this.urls.length; ++i) {
                if (this.urls[i] == url) {
                    return true;
                }
            }
            return false;
        }
    }
    
    private class StartupClassLoader extends URLClassLoader
    {
        public StartupClassLoader(final URL[] urls) {
            super(urls);
        }
        
        public StartupClassLoader(final URL[] urls, final ClassLoader parent) {
            super(urls, parent);
        }
        
        public StartupClassLoader(final URL[] urls, final ClassLoader parent, final URLStreamHandlerFactory factory) {
            super(urls, parent, factory);
        }
        
        @Override
        protected String findLibrary(final String name) {
            if (Main.this.extensionPaths == null) {
                return super.findLibrary(name);
            }
            final String libName = System.mapLibraryName(name);
            for (int i = 0; i < Main.this.extensionPaths.length; ++i) {
                final File libFile = new File(Main.this.extensionPaths[i], libName);
                if (libFile.isFile()) {
                    return libFile.getAbsolutePath();
                }
            }
            return super.findLibrary(name);
        }
    }
}
